from __future__ import absolute_import

import base64

from Cryptodome import Random
from Cryptodome.Cipher import PKCS1_v1_5
from Cryptodome.Hash import SHA
from Cryptodome.Hash import SHA256
from Cryptodome.PublicKey import RSA
from Cryptodome.Signature import PKCS1_v1_5 as signPKC
from six import string_types

# from .logger_util import LoggerUtil
from .file_util import FileUtil
from .properties_file_reader import PropertiesFileReader
from ..configuration import Configuration
from ..exceptions import ApiException


class EncryptionUtil(object):
    ENCODE_TYPE = 'utf-8'
    # logger = LoggerUtil.setup_logger(__name__)
    _private_key = None
    _checkout_public_key = None
    checkout_ssl_cert_finger_print = None

    @classmethod
    def getAllProperties(cls):
        try:
            private_key_data = FileUtil.read_file(PropertiesFileReader.get_property(PropertiesFileReader.CLIENT_PRIVATE_KEY_LOCATION))
            public_key_data = FileUtil.read_file(PropertiesFileReader.get_property(PropertiesFileReader.CHECKOUT_PUBLIC_KEY_LOCATION))

            EncryptionUtil._private_key = RSA.importKey(
                private_key_data,
                passphrase=PropertiesFileReader.get_property(PropertiesFileReader.CLIENT_PRIVATE_KEY_PASSWORD))
            EncryptionUtil._checkout_public_key = RSA.importKey(public_key_data)
            if Configuration().verify_ssl:
                checkout_ssl_cert_data = FileUtil.read_file(
                    PropertiesFileReader.get_property(PropertiesFileReader.CHECKOUT_TLS_CERTIFICATE_LOCATION))
                _hash = SHA256.new()
                _hash.update(checkout_ssl_cert_data)
                EncryptionUtil.checkout_ssl_cert_finger_print = _hash.hexdigest()

        except BaseException as e:  # logger.error("exception occured: {}".format(e))
            raise ApiException(
                status=0,
                reason="Failed to load keys {}".format(e)
            )

    @classmethod
    def encrypt(cls, plain_text):
        if (EncryptionUtil._checkout_public_key is None):
            EncryptionUtil.getAllProperties()

        try:
            if isinstance(plain_text, string_types):
                cipher = PKCS1_v1_5.new(EncryptionUtil._checkout_public_key)
                encrypted_text = cipher.encrypt(plain_text.encode(EncryptionUtil.ENCODE_TYPE))
                return base64.b64encode(encrypted_text).decode(EncryptionUtil.ENCODE_TYPE)
            else:
                raise TypeError("plain_text must be a string")
        except BaseException as e:
            # EncryptionUtil.logger.error("exception occured: {}".format(e))
            raise ApiException(
                status=0,
                reason="Failed to encrypt {}".format(e)
            )

    @classmethod
    def decrypt(cls, encrypted_text):
        # EncryptionUtil.logger.info('in decrypt')
        if (EncryptionUtil._private_key is None):
            EncryptionUtil.getAllProperties()

        try:
            if isinstance(encrypted_text, string_types):
                dsize = SHA.digest_size
                sentinel = Random.new().read(15 + dsize)
                cipher = PKCS1_v1_5.new(EncryptionUtil._private_key)
                plain_text = cipher.decrypt(base64.b64decode(encrypted_text), sentinel)
                return plain_text.decode(EncryptionUtil.ENCODE_TYPE)
            else:
                raise TypeError("encrypted_text must be a string")
        except BaseException as e:
            # EncryptionUtil.logger.error("exception occured: {}".format(e))
            raise ApiException(
                status=0,
                reason="Failed to decrypt {}".format(e)
            )

    @classmethod
    def sign(cls, plain_text):
        if (EncryptionUtil._private_key is None):
            EncryptionUtil.getAllProperties()

        try:
            if isinstance(plain_text, string_types):
                signer = signPKC.new(EncryptionUtil._private_key)
                hashed_text = SHA.new(plain_text.encode(EncryptionUtil.ENCODE_TYPE))
                signature = signer.sign(hashed_text)
                signature = base64.b64encode(signature).decode(EncryptionUtil.ENCODE_TYPE)
                return signature
            else:
                raise TypeError("plain_text must be a string")
        except BaseException as e:
            # EncryptionUtil.logger.error("exception occured: {}".format(e))
            raise ApiException(
                status=0,
                reason="Failed to sign {}".format(e)
            )

    @classmethod
    def verify(cls, plain_text, signature):
        if (EncryptionUtil._checkout_public_key is None):
            EncryptionUtil.getAllProperties()

        try:
            if isinstance(plain_text, string_types) and isinstance(signature, string_types):
                signature_bytes = base64.b64decode(signature.encode(EncryptionUtil.ENCODE_TYPE))
                hashed_text = SHA.new(plain_text.encode(EncryptionUtil.ENCODE_TYPE))
                verifier = signPKC.new(EncryptionUtil._checkout_public_key)
                return verifier.verify(hashed_text, signature_bytes)
            else:
                raise TypeError("plain_text and signature must be strings")
        except BaseException as e:
            # EncryptionUtil.logger.error("exception occured: {}".format(e))
            raise ApiException(
                status=0,
                reason="Failed to verify {}".format(e)
            )
